"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const consul_1 = __importDefault(require("consul"));
const fs_1 = require("fs");
const js_yaml_1 = __importDefault(require("js-yaml"));
function parseString(str) {
    if (str === "true")
        return true;
    if (str === "false")
        return false;
    if (str === "null")
        return null;
    if (str === "undefined")
        return undefined;
    const num = Number(str);
    if (!isNaN(num))
        return num;
    return str;
}
async function getConsulConfigNested(prefix) {
    if (!prefix)
        return undefined;
    const consulClient = consul_1.default({ promisify: true });
    const keys = await consulClient.kv.keys(process.env.CONSUL_KV_PREFIX);
    const values = (await Promise.all(keys.map((key) => consulClient.kv.get(key))));
    return values.reduce((obj, value, i) => {
        const { Key, Value } = value;
        const fullPath = Key.replace(prefix, "");
        const path = fullPath.split("/").filter((s) => s !== "");
        if (!path.length)
            return obj;
        const basePath = path.splice(0, path.length - 1);
        const [fieldName] = path;
        const base = basePath.reduce((x, k) => x[k], obj);
        if (Value === null) {
            base[fieldName] = {};
        }
        else {
            base[fieldName] = parseString(Value);
        }
        return obj;
    }, {});
}
async function getConsulConfigJson(key) {
    if (!key)
        return undefined;
    const consulClient = consul_1.default({ promisify: true });
    const result = await consulClient.kv.get(key);
    return JSON.parse(result.Value);
}
async function getConfig(configVars) {
    var _a, _b, _c;
    const path = process.env.CONFIG_PATH || (configVars === null || configVars === void 0 ? void 0 : configVars.configPath);
    const fileConfig = path
        ? js_yaml_1.default.safeLoad(fs_1.readFileSync(path, "utf8"))
        : undefined;
    const consulConfigNested = await getConsulConfigNested(process.env.CONSUL_KV_PREFIX);
    const consulConfigJson = await getConsulConfigJson(process.env.CONSUL_KV_KEY);
    const mergedConfig = Object.assign({}, fileConfig, consulConfigNested, consulConfigJson, configVars);
    const mqttUri = process.env.MQTT_URI || ((_a = mergedConfig.mqtt) === null || _a === void 0 ? void 0 : _a.uri);
    if (!mqttUri) {
        throw new Error("MQTT URI Required.");
    }
    const mqttPrefix = process.env.MQTT_PREFIX || ((_b = mergedConfig.mqtt) === null || _b === void 0 ? void 0 : _b.prefix) || "";
    const httpPort = (process.env.HTTP_PORT && parseInt(process.env.HTTP_PORT)) || ((_c = mergedConfig.http) === null || _c === void 0 ? void 0 : _c.port) ||
        null;
    const mqtt = Object.assign(Object.assign({ subscriptions: [] }, mergedConfig.mqtt), { uri: mqttUri, prefix: mqttPrefix });
    const http = Object.assign(Object.assign({}, mergedConfig.http), { port: httpPort });
    const service = mergedConfig.service || {};
    return {
        mqtt,
        http,
        service,
    };
}
exports.getConfig = getConfig;
//# sourceMappingURL=config.js.map