"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const mqtt_1 = require("./mqtt");
const events_1 = require("events");
const metrics_1 = require("./metrics");
function createService(mqttConfig, serviceConfig, logger) {
    const { prefix, subscriptions } = mqttConfig;
    const e = new events_1.EventEmitter();
    const topic = (key) => key.startsWith("~") ? key.replace("~", prefix) : key;
    const mqtt = mqtt_1.startMqtt(mqttConfig, logger);
    const service = {
        config: serviceConfig,
        prefix: prefix,
        get mqttConnected() {
            return mqtt.connected;
        },
        send(key, data, options) {
            mqtt.client.publish(topic(key), JSON.stringify(data), options);
            metrics_1.messagesSentCounter.inc();
        },
        subscribe(key) {
            mqtt.client.subscribe(topic(key));
        },
        end() {
            mqtt.client.end();
        },
        on(event, listener) {
            e.on(event, listener);
        },
        removeListener(event, listener) {
            e.removeListener(event, listener);
        },
        removeAllListeners(event) {
            e.removeAllListeners(event);
        },
    };
    subscriptions.forEach((s) => service.subscribe(s));
    mqtt.client.on("message", (topic, message) => {
        metrics_1.messagesReceivedCounter.inc();
        // message is Buffer
        const str = message.toString();
        let data;
        try {
            data = JSON.parse(str);
        }
        catch (err) {
            // naive error handling
            data = str;
        }
        if (topic.startsWith(prefix)) {
            e.emit("message", topic.replace(prefix, "~"), data);
            return;
        }
        e.emit("message", topic, data);
    });
    return service;
}
exports.createService = createService;
//# sourceMappingURL=service.js.map